package com.winhc.bigdata.task.configuration;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author: XuJiakai
 * 2021/4/14 15:24
 */
@Configuration
public class ThreadPoolExecutorConfig {
    /**
     * 线程池维护线程的最少数量
     */
    @Value("${thread.pool.core.pool.size:1}")
    private int threadPoolCorePoolSize;
    /**
     * 线程池维护线程的最大数量
     */
    @Value("${thread.pool.max.pool.size:1}")
    private int threadPoolMaxPoolSize;
    /**
     * 缓存队列
     */
    @Value("${thread.pool.queue.capacity:10}")
    private int threadPoolQueueCapacity;
    /**
     * 允许的空闲时间
     */
    @Value("${thread.pool.keep.alive.seconds:300}")
    private int threadPoolKeepAliveSeconds;


    @Bean(name = "export-data-pool")
    public ThreadPoolTaskExecutor getMyThreadPoolExecutor() {
        ThreadPoolTaskExecutor taskExecutor = new ThreadPoolTaskExecutor();

        taskExecutor.setCorePoolSize(threadPoolCorePoolSize);
        taskExecutor.setMaxPoolSize(threadPoolMaxPoolSize);
        taskExecutor.setQueueCapacity(threadPoolQueueCapacity);
        taskExecutor.setKeepAliveSeconds(threadPoolKeepAliveSeconds);
        //定义线程池名字
        taskExecutor.setThreadNamePrefix("export-data-pool");
        //对拒绝task的处理策略
        taskExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        taskExecutor.initialize();
        return taskExecutor;
    }
}
