package com.winhc.bigdata.task.configuration;

import com.winhc.bigdata.task.service.DingTalkService;
import com.winhc.bigdata.task.vo.ResponseVo;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import javax.validation.ConstraintViolationException;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.INTERNAL_SERVER_ERROR;

/**
 * @Author: XuJiakai
 * @Date: 2020/8/7 11:03
 * @Description:
 */
@Slf4j
@Order(Ordered.HIGHEST_PRECEDENCE)
@ControllerAdvice
@AllArgsConstructor
public class ValidationAdvice {
    private final DingTalkService dingTalkService;

    @ResponseStatus(BAD_REQUEST)
    @ResponseBody
    @ExceptionHandler(ConstraintViolationException.class)
    public ResponseVo<String> methodArgumentNotValidException(ConstraintViolationException ex) {
        log.error(ex.getMessage(), ex);
        dingTalkService.error(ex.getMessage());
        return ResponseVo.failure(ex.getMessage());
    }


    @ResponseStatus(BAD_REQUEST)
    @ResponseBody
    @ExceptionHandler(IllegalArgumentException.class)
    public ResponseVo<String> illegalArgumentException(IllegalArgumentException ex) {
        log.error(ex.getMessage(), ex);
        dingTalkService.error(ex.getMessage());
        return ResponseVo.failure(ex.getMessage());
    }

    @ResponseStatus(BAD_REQUEST)
    @ResponseBody
    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public ResponseVo<String> httpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException ex) {
        log.error(ex.getMessage(), ex);
        return ResponseVo.failure(ex.getMessage());
    }


    @ResponseStatus(INTERNAL_SERVER_ERROR)
    @ResponseBody
    @ExceptionHandler(Exception.class)
    public ResponseVo<String> exception(Exception ex) {
        log.error(ex.getMessage(), ex);
        dingTalkService.error(ex.getMessage());
        return ResponseVo.failure(ex.getMessage());
    }
}
