package com.winhc.bigdata.task.framework.odps.handle;

import cn.hutool.core.util.StrUtil;
import com.aliyun.odps.Instance;
import com.winhc.bigdata.task.enums.TaskStatusEnum;
import com.winhc.bigdata.task.framework.odps.AsyncTaskHandle;
import com.winhc.bigdata.task.framework.odps.entity.MaxComputeInstance;
import com.winhc.bigdata.task.framework.odps.entity.OdpsResultInfo;
import com.winhc.bigdata.task.framework.odps.exception.AsyncTaskRunningException;
import com.winhc.bigdata.task.framework.odps.mapper.MaxComputeInstanceMapper;
import com.winhc.bigdata.task.framework.odps.service.OdpsResultInfoService;
import com.winhc.bigdata.task.framework.odps.service.OdpsSqlInfoService;
import com.winhc.bigdata.task.framework.odps.utils.ExecOdpsSqlUtil;
import com.winhc.bigdata.task.framework.odps.vo.MaxComputeSqlInstance;
import com.winhc.bigdata.task.framework.odps.vo.OdpsSqlInfoVO;
import com.winhc.bigdata.task.util.GenerateIdUtils;
import com.winhc.bigdata.task.util.JsonUtils;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/7/5 16:49
 */
@Slf4j
@Service("sql_task")
@AllArgsConstructor
public class SqlTaskHandle implements AsyncTaskHandle {
    private final OdpsSqlInfoService sqlInfoService;
    private final ExecOdpsSqlUtil execOdpsSqlUtil;
    private final MaxComputeInstanceMapper mapper;
    private final OdpsResultInfoService odpsResultInfoService;

    @Override
    public String run(String taskId, String operatorName) throws AsyncTaskRunningException {
        Boolean lock = sqlInfoService.lock(taskId, operatorName);
        if (!lock) {
            throw new AsyncTaskRunningException("存在运行中的任务");
        }
        try {
            OdpsSqlInfoVO odpsSqlInfoVO = sqlInfoService.get(taskId);
            HashMap<String, String> map = new HashMap<>(3);
            String randomId = GenerateIdUtils.generateTaskId();
            map.put("random", randomId);

            String sql = StrUtil.format(odpsSqlInfoVO.getSql(), map);

            MaxComputeSqlInstance exec = execOdpsSqlUtil.exec(sql);

            MaxComputeInstance maxComputeInstance = MaxComputeInstance.builder()
                    .id(exec.getId())
                    .sqlId(taskId)
                    .type("sql_task")
                    .detail(JsonUtils.jsonObjToString(exec))
                    .successful(false)
                    .instanceTerminated(false)
                    .build();
            mapper.insert(maxComputeInstance);

            List<OdpsResultInfo> list = new ArrayList<>();
            for (Integer i = 1; i <= odpsSqlInfoVO.getPartitions(); i++) {
                OdpsResultInfo sql_task = OdpsResultInfo.builder()
                        .sqlId(taskId)
                        .instanceId(exec.getId())
                        .instanceType("sql_task")
                        .tableName(odpsSqlInfoVO.getOutputTab())
                        .partitionName(randomId + "_" + i)
                        .recordCount(-1L)
                        .priorityLevel(i)
                        .partitionTaskStatus(TaskStatusEnum.生成中)
                        .build();
                list.add(sql_task);
            }

            odpsResultInfoService.saveBatch(list);
            log.info("执行sql:{} 执行人：{}", taskId, operatorName);
            return exec.getId();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @SneakyThrows
    @Override
    public Boolean isSuccess(String instanceId) {
        MaxComputeInstance maxComputeInstance = mapper.selectById(instanceId);
        if (maxComputeInstance.getInstanceTerminated()) {
            return maxComputeInstance.getSuccessful();
        }
        Instance instance = execOdpsSqlUtil.getInstance(instanceId);
        MaxComputeSqlInstance of = MaxComputeSqlInstance.of(instance);
        if (of.getTerminated()) {


            MaxComputeInstance task = MaxComputeInstance.builder()
                    .id(of.getId())
                    .sqlId(maxComputeInstance.getSqlId())
                    .type("sql_task")
                    .detail(JsonUtils.jsonObjToString(of))
                    .successful(of.getSuccessful())
                    .instanceTerminated(of.getTerminated())
                    .build();
            mapper.updateById(task);
            // fixme 更新结果表
            if (of.getSuccessful()) {
                log.info("sqlId: {} 执行成功 ！", maxComputeInstance.getSqlId());
                odpsResultInfoService.lambdaUpdate()
                        .eq(OdpsResultInfo::getInstanceId, instanceId)
                        .set(OdpsResultInfo::getPartitionTaskStatus, TaskStatusEnum.生成完毕)
                        .update();
            } else {
                List<OdpsResultInfo> list = odpsResultInfoService.lambdaQuery().eq(OdpsResultInfo::getInstanceId, instanceId).list();
                List<Long> collect = list.stream().map(OdpsResultInfo::getId).collect(Collectors.toList());
                odpsResultInfoService.removeByIds(collect);
                log.error("sqlId: {} 执行失败 ！", maxComputeInstance.getSqlId());
            }
            sqlInfoService.unlock(maxComputeInstance.getSqlId());
            return of.getSuccessful();
        } else {
            return false;
        }
    }
}
