package com.winhc.bigdata.task.framework.odps.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.bigdata.task.framework.odps.entity.OdpsSqlInfo;
import com.winhc.bigdata.task.framework.odps.mapper.OdpsSqlInfoMapper;
import com.winhc.bigdata.task.framework.odps.service.OdpsSqlInfoService;
import com.winhc.bigdata.task.framework.odps.utils.OdpsSchemaUtils;
import com.winhc.bigdata.task.framework.odps.vo.OdpsSqlInfoVO;
import com.winhc.bigdata.task.framework.odps.vo.RegisterSqlVO;
import com.winhc.bigdata.task.framework.odps.vo.SortVO;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/7/5 16:20
 */
@Slf4j
@Service
@AllArgsConstructor
public class OdpsSqlInfoServiceImpl extends ServiceImpl<OdpsSqlInfoMapper, OdpsSqlInfo> implements OdpsSqlInfoService {
    private final OdpsSchemaUtils odpsSchemaUtils;

    @Override
    public Boolean lock(String sqlId, String lockByName) {
        OdpsSqlInfo byId = getById(sqlId);
        Assert.notNull(byId, "sql id 不存在！");
        if (byId.getLockSign()) {
            log.error("sqlId: {} lock failure !", sqlId);
            return false;
        } else {
            byId.setLockSign(true);
            byId.setLockByName(lockByName);
            updateById(byId);
            log.info("sqlId: {} lock successful !", sqlId);
            return true;
        }
    }

    @Override
    public Boolean unlock(String sqlId) {
        OdpsSqlInfo byId = getById(sqlId);
        Assert.notNull(byId, "sql id 不存在！");
        if (byId.getLockSign()) {
            byId.setLockSign(false);
            byId.setLockByName(null);
            updateById(byId);
            log.info("sqlId: {} unlock successful !", sqlId);
            return true;
        } else {
            log.warn("sqlId: {} unlock. but sql is unlocked !", sqlId);
            return false;
        }
    }

    private static final String sqlTemp = "INSERT OVERWRITE TABLE winhc_ng.out_pull_spider_data PARTITION(ds) SELECT to_json(Map({maps})) AS detail_json ,CONCAT_ws('_','{random}',rn) AS ds FROM ( SELECT * ,ntile({partitionNum}) OVER({orderBy}) rn FROM ( {allTab} ) ) ;";

    private void checkColumns(List<String> exportColumns, String tn) {
        List<String> allColumns = odpsSchemaUtils.getAllColumns("winhc_ng", "inc_ads_" + tn);
        allColumns.retainAll(exportColumns);
        Assert.isTrue(allColumns.size() == exportColumns.size(), "要导出的列不存在 ！");
    }

    @Override
    public OdpsSqlInfoVO registerSql(RegisterSqlVO registerSqlVO) {
        String tn = registerSqlVO.getTn();
        String json = registerSqlVO.getColumns().stream().map(r -> "\"" + r + "\"," + r).collect(Collectors.joining(","));
        Integer partitionNum = registerSqlVO.getPartitionNum();
        String registerBy = registerSqlVO.getRegisterBy();

        checkColumns(registerSqlVO.getColumns(), tn);

        String allSql = odpsSchemaUtils.getAllSql(tn, "winhc_ng", "ads", registerSqlVO.getWhere());

        Map<String, String> map = new HashMap<String, String>(12) {
            {
                put("maps", json);
                put("partitionNum", partitionNum + "");
                put("allTab", allSql);
            }
        };

        List<SortVO> sort = registerSqlVO.getSort();
        String orderBy = "";
        if (!sort.isEmpty()) {
            orderBy = sort.stream().map(r -> {
                if (r.getAse()) {
                    return r.getField() + " ase";
                } else {
                    return r.getField() + " desc";
                }
            }).collect(Collectors.joining(",", "ORDER BY ", ""));
        }
        map.put("orderBy", orderBy);

        String sql = StrUtil.format(sqlTemp, map);
        OdpsSqlInfoVO odpsSqlInfoVO = OdpsSqlInfoVO.builder()
                .sql(sql)
                .outputTab("out_pull_spider_data")
                .createBy(registerBy)
                .partitions(partitionNum)
                .build();
        add(odpsSqlInfoVO);
        return odpsSqlInfoVO;
    }

    private static final Pattern pattern = Pattern.compile("[\r\n ]+");

    @Override
    public String add(OdpsSqlInfoVO sqlInfo) {
        String sql = pattern.matcher(sqlInfo.getSql()).replaceAll(" ").trim();
        Assert.isTrue(StringUtils.isNotEmpty(sql), "sql为空！");
        if (StringUtils.isEmpty(sqlInfo.getId())) {
            sqlInfo.setId(DigestUtil.md5Hex(sql));
        }
        OdpsSqlInfo build = OdpsSqlInfo.builder()
                .id(sqlInfo.getId())
                .sqlStr(sql)
                .createBy(sqlInfo.getCreateBy())
                .outputTab(sqlInfo.getOutputTab())
                .partitions(sqlInfo.getPartitions()).build();
        saveOrUpdate(build);
        return build.getId();
    }

    @Override
    public OdpsSqlInfoVO get(String sqlId) {
        OdpsSqlInfo byId = getById(sqlId);
        return OdpsSqlInfoVO.builder()
                .id(byId.getId())
                .sql(byId.getSqlStr())
                .outputTab(byId.getOutputTab())
                .partitions(byId.getPartitions())
                .createBy(byId.getCreateBy())
                .build();

    }

    @Override
    public Boolean check(String sqlId) {
        return getById(sqlId) != null;
    }
}
