package com.winhc.bigdata.task.framework.odps.utils;

import cn.hutool.core.util.StrUtil;
import com.aliyun.odps.Column;
import com.aliyun.odps.Odps;
import com.aliyun.odps.Table;
import com.aliyun.odps.TableSchema;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/7/7 09:44
 */
@Component
@AllArgsConstructor
public class OdpsSchemaUtils {
    private final Odps odps;

    public List<String> getColumns(String projectName, String tableName) {
        Table table = odps.tables().get(projectName, tableName);
        TableSchema schema = table.getSchema();
        List<Column> columns = schema.getColumns();
        return columns.stream().map(Column::getName).collect(Collectors.toList());
    }

    public List<String> getPartitionColumns(String projectName, String tableName) {
        Table table = odps.tables().get(projectName, tableName);
        TableSchema schema = table.getSchema();
        return schema.getPartitionColumns().stream().map(Column::getName).collect(Collectors.toList());
    }

    public List<String> getAllColumns(String projectName, String tableName) {
        List<String> columns = getColumns(projectName, tableName);
        columns.addAll(getPartitionColumns(projectName, tableName));
        return columns;
    }

    private final String sqlTemp = "SELECT {cols} FROM ( SELECT * ,ROW_NUMBER() OVER(PARTITION BY {rowkey} ORDER BY ds DESC ) AS num FROM ( SELECT {cols} FROM {project}.{allTab} WHERE ds > 0 UNION ALL SELECT {cols} FROM {project}.{incTab} WHERE ds > 0 ) AS all_t1 ) AS all_t2 WHERE all_t2.num = 1 {where}";

    public String getAllSql(String tn, String project, String prefix,String whereExpression) {
        String allTab = prefix + "_" + tn;
        String incTab = "inc_" + prefix + "_" + tn;

        List<String> columns = getAllColumns(project, incTab);
        columns.retainAll(getAllColumns(project, allTab));
        String rowkey;
        if (columns.contains("rowkey")) {
            rowkey = "rowkey";
        } else if (columns.contains("company_id")) {
            rowkey = "company_id";
        } else {
            throw new RuntimeException("PARTITION BY 找不到对应的值 !");
        }

        String where = "";
        if (StringUtils.isNotEmpty(whereExpression)) {
            where = " AND (" + whereExpression + ")";
        }
        String finalWhere = where;
        Map<String, String> map = new HashMap<String, String>() {
            {
                put("cols", String.join(",", columns));
                put("allTab", allTab);
                put("incTab", incTab);
                put("project", project);
                put("rowkey", rowkey);
                put("where", finalWhere);
            }
        };
        String format = StrUtil.format(sqlTemp, map);
        return format;
    }
}
