package com.winhc.bigdata.task.jobs;

import com.mongodb.client.MongoCollection;
import com.winhc.bigdata.task.service.DingTalkService;
import com.winhc.bigdata.task.util.EsFastScan;
import com.winhc.bigdata.task.util.MaxBatchQueryUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.bson.Document;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/6/18 16:18
 */
@Slf4j
@Component
@AllArgsConstructor
public class RemoveCompanyIdLookupJob {
    private final DingTalkService dingTalkService;
    private final RestHighLevelClient restHighLevelClient;
    private final MongoTemplate mongoTemplate;
    private final Table2CompanyIdMapProperties table2CompanyIdMapProperties;


    @KafkaListener(id = "RemoveCompanyIdLookupJob"
            , topics = "inc_update_dimension"
            , groupId = "${spring.kafka.consumer.group-id:xjk_group}", containerFactory = "containerFactory")
    public void removeCompanyIdLookupJob(List<ConsumerRecord<String, String>> records) {
        List<String> collect = records.stream().map(ConsumerRecord::value).collect(Collectors.toList());
        exec(collect);
    }


    public Boolean exec(List<String> companyId) {
        for (String key : table2CompanyIdMapProperties.getMap().keySet()) {
            try {
                exec(companyId, key);
            } catch (Exception e) {
                MongoCollection<Document> error = mongoTemplate.getCollection("RemoveCompanyIdLookupJob_error");
                List<Document> rowkey = companyId.stream().map(r -> {
                    Document document = new Document();
                    document.put("company_id", r);
                    document.put("tn", key);
                    return document;
                }).collect(Collectors.toList());
                error.insertMany(rowkey);
                log.error(e.getMessage(), e);
                dingTalkService.error("RemoveCompanyIdLookupJob {}", e.getMessage());
            }
        }
        return true;
    }


    public Boolean exec(List<String> companyIds, String tn) {
        if (!table2CompanyIdMapProperties.getMap().containsKey(tn)) {
            dingTalkService.error("RemoveCompanyIdLookupJob 移除公司关联id查找，存在未知tn: {}", tn);
            return false;
        }
        if (companyIds.isEmpty()) {
            return true;
        }
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
        String s = table2CompanyIdMapProperties.getMap().get(tn);
        if (s.contains(",")) {
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            for (String k : s.split(",")) {
                boolQueryBuilder.should(MaxBatchQueryUtils.addTerms(companyIds, k));
            }
            boolQuery.filter(boolQueryBuilder);
        } else {
            boolQuery.filter(MaxBatchQueryUtils.addTerms(companyIds, s));
        }


        MongoCollection<Document> tab = mongoTemplate.getCollection("company_back_update_0423");
        Consumer<SearchHit[]> func = list -> {
            List<Document> li = Arrays.stream(list).map(d -> {
                String id = d.getId();
                Document document = new Document();
                document.put("rowkey", id);
                document.put("tn", tn);
                return document;
            }).collect(Collectors.toList());
            tab.insertMany(li);
        };
        new EsFastScan(restHighLevelClient, func, "winhc_index_" + tn, "_doc", boolQuery).scan();
        return true;
    }
}
