package com.winhc.bigdata.task.service.impl;

import com.winhc.bigdata.task.dao.CompanyIndexDao;
import com.winhc.bigdata.task.service.CompanyIndexService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.range.ParsedRange;
import org.elasticsearch.search.aggregations.bucket.range.RangeAggregationBuilder;
import org.elasticsearch.search.aggregations.bucket.range.RangeAggregator;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.stream.Collectors;

import static org.elasticsearch.index.query.QueryBuilders.matchAllQuery;

/**
 * @author: XuJiakai
 * 2021/4/13 14:28
 */
@Slf4j
@Service
@AllArgsConstructor
public class CompanyIndexServiceImpl implements CompanyIndexService {
    private final CompanyIndexDao companyIndexDao;


    @Override
    public Map<String, Long> getDataDistribution() {
        RangeAggregationBuilder agg = AggregationBuilders.range("score_agg").field("company_score_weight");
        agg.addRange(new RangeAggregator.Range("0", null, 1.0));
        agg.addRange("1", 1, 2);
        agg.addRange("2", 2, 3);
        agg.addRange("3", 3, 4);
        agg.addRange("4", 4, 5);
        agg.addRange("5", 5, 6);
        agg.addRange("6", 6, 7);
        agg.addRange("7", 7, 8);
        agg.addRange("8", 8, 9);
        agg.addRange("9", 9, 10);
        agg.addRange(new RangeAggregator.Range("10", 10.0, null));


        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder().query(matchAllQuery()).size(0)
                .aggregation(agg);

        SearchResponse search = companyIndexDao.search(sourceBuilder);
        ParsedRange scoreAgg = search.getAggregations().get("score_agg");

        return scoreAgg.getBuckets()
                .stream()
                .map(o -> (ParsedRange.ParsedBucket) o)
                .collect(Collectors.toMap(ParsedRange.ParsedBucket::getKeyAsString, ParsedRange.ParsedBucket::getDocCount));
    }
}
