package com.winhc.bigdata.task.service.impl;

import cn.hutool.core.codec.Base64;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagRequest;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceRequest;
import com.aliyuncs.dataworks_public.model.v20180601.SearchManualDagNodeInstanceResponse;
import com.aliyuncs.http.ProtocolType;
import com.winhc.bigdata.task.configuration.DataWorksAccessProperties;
import com.winhc.bigdata.task.configuration.ProjectParamInit;
import com.winhc.bigdata.task.enums.DataWorksTaskFlowEnum;
import com.winhc.bigdata.task.service.CompanyIndexService;
import com.winhc.bigdata.task.service.DataWorksService;
import com.winhc.bigdata.task.util.DateUtils;
import com.winhc.bigdata.task.util.JsonUtils;
import com.winhc.bigdata.task.vo.dataworks.DataWorksFlowTask;
import com.winhc.bigdata.task.vo.dataworks.TaskParam;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/4/14 14:10
 */
@Slf4j
@Service
@AllArgsConstructor
public class DataWorksServiceImpl implements DataWorksService {

    private final DataWorksAccessProperties dataWorksAccessProperties;
    private final IAcsClient client;
    private final Map<String, DataWorksFlowTask> dataWorksFlowTaskMap;
    private final CompanyIndexService companyIndexService;

    @SneakyThrows
    public CreateManualDagResponse touch(String projectName
            , String flowName, String bizDate) {
        return triggerWithParam(projectName, flowName, bizDate, null, null);
    }

    @SneakyThrows
    public CreateManualDagResponse triggerWithParam(String projectName
            , String flowName, String bizDate, String dagPara, String nodePara) {
        log.info("触发任务：{}.{} {}\n{}\n{}", projectName, flowName, bizDate,dagPara,nodePara);
        CreateManualDagRequest request = new CreateManualDagRequest();
        request.setProjectName(projectName);
        request.setFlowName(flowName);
        request.setBizdate(bizDate + " 00:00:00");
        request.setRegionId(dataWorksAccessProperties.getRegionId());
        request.setProtocol(ProtocolType.HTTP);
        if (StringUtils.isNotEmpty(dagPara)) {
            request.setDagPara(dagPara);
        }
        if (StringUtils.isNotEmpty(nodePara)) {
            request.setNodePara(nodePara);
        }
        CreateManualDagResponse response = client.getAcsResponse(request);
        log.info("\n任务结果：\n\trequestId:{},\n\treturnCode:{},\n\treturnErrorSolution:{},\n\treturnMessage:{},\n\treturnValue:{}",
                response.getRequestId()
                , response.getReturnCode()
                , response.getReturnErrorSolution()
                , response.getReturnMessage()
                , response.getReturnValue());
        return response;
    }

    @SneakyThrows
    public CreateManualDagResponse touch(TaskParam taskParam) {
        return triggerWithParam(taskParam.getProjectName(), taskParam.getFlowName(), taskParam.getBizDate(), null, JsonUtils.jsonObjToString(taskParam.getNodeParam()));
    }


    @Override
    public Long exportData(String dt) {
        String yesterday = DateUtils.getYesterday();
        HashMap<String, String> map = new HashMap<>();
        map.put("dt", dt);
        return touch(ProjectParamInit.project, ProjectParamInit.exportFlowName, yesterday, dataWorksFlowTaskMap.get(ProjectParamInit.exportBeanName), map);
    }

    @Override
    public Long generateData(String taskId) {
        Map<String, Long> dataDistribution = companyIndexService.getDataDistribution();
        String s = JsonUtils.jsonObjToString(dataDistribution);
        String yesterday = DateUtils.getYesterday();
        HashMap<String, String> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("dataDistribution", Base64.encode(s));
        return touch(ProjectParamInit.project, ProjectParamInit.generateFlowName, yesterday, dataWorksFlowTaskMap.get(ProjectParamInit.generateBeanName), map);
    }

    @Override
    public Long touch(String project, String flowName, String bizDate, DataWorksFlowTask dataWorksFlowTask, Map<String, String> otherParams) {
        TaskParam build = TaskParam.builder()
                .projectName(project)
                .bizDate(bizDate)
                .flowName(flowName)
                .nodeParam(dataWorksFlowTask.toNodeParam(bizDate, otherParams))
                .build();
        CreateManualDagResponse touch = touch(build);
        return touch.getReturnValue();
    }

    @SneakyThrows
    @Override
    public Map<String, DataWorksTaskFlowEnum> query(String projectName, Long dagId) {
        SearchManualDagNodeInstanceRequest searchNodeInstanceListRequest
                = new SearchManualDagNodeInstanceRequest();
        searchNodeInstanceListRequest.setDagId(dagId);
        searchNodeInstanceListRequest.setProjectName(projectName);
        searchNodeInstanceListRequest.setProtocol(ProtocolType.HTTP);
        SearchManualDagNodeInstanceResponse searchResponse = client
                .getAcsResponse(searchNodeInstanceListRequest);
        java.util.List<SearchManualDagNodeInstanceResponse.NodeInsInfo> nodeInsfos = searchResponse.getData();
        for (SearchManualDagNodeInstanceResponse.NodeInsInfo nodeInsInfo : nodeInsfos) {
            DataWorksTaskFlowEnum code = DataWorksTaskFlowEnum.getTaskFlowEnumByCode(nodeInsInfo.getStatus());
            log.info("{}:{} {}", nodeInsInfo.getNodeName(), nodeInsInfo.getStatus(), code);
        }
        return nodeInsfos.stream()
                .collect(Collectors.toMap(SearchManualDagNodeInstanceResponse.NodeInsInfo::getNodeName, node -> DataWorksTaskFlowEnum.getTaskFlowEnumByCode(node.getStatus()), (o1, o2) -> o1));

    }
}
