package com.winhc.bigdata.task.service.impl;

import cn.hutool.core.util.StrUtil;
import com.winhc.bigdata.task.constant.DingTalkConst;
import com.winhc.bigdata.task.service.DingTalkService;
import com.winhc.bigdata.task.util.DingTalkUtils;
import com.winhc.bigdata.task.util.OkHttpUtils;
import com.winhc.bigdata.task.util.ding.talk.DingTalkText;
import com.winhc.bigdata.task.vo.Entry;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author: XuJiakai
 * 2021/4/12 14:05
 */
@Slf4j
@Service
@AllArgsConstructor
public class DingTalkServiceImpl implements DingTalkService {
    private OkHttpUtils client;
    private static final String URL = "https://oapi.dingtalk.com/robot/send?access_token=";

    @Override
    public boolean send(String text) {
        return send(DingTalkText.builder().content(text).build().packString(), DingTalkConst.winhc_log_talk);
    }

    @Override
    public boolean info(String infoMsgTemplate, Object... params) {
        return send("【winhc-bigdata-task.info】" + StrUtil.format(infoMsgTemplate, params));
    }

    @Override
    public boolean warn(String warnMsgTemplate, Object... params) {
        return send("【winhc-bigdata-task.warn】" + StrUtil.format(warnMsgTemplate, params));
    }

    @Override
    public boolean error(String errorMsgTemplate, Object... params) {
        return send("【winhc-bigdata-task.error】" + StrUtil.format(errorMsgTemplate, params));
    }

    private boolean send(String body, DingTalkConst to) {
        try {
            Entry<Long, String> sign = DingTalkUtils.getSign(to.getDingSecret());
            String query = to.getAccessToken() + "&timestamp=" + sign.getKey() + "&sign=" + sign.getValue();
            String post = client.post(URL + query, body);
            log.info(post);
            return post.contains("ok");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return false;
        }
    }
}
