package com.winhc.bigdata.task.util;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author: XuJiakai
 * 2021/5/28 17:05
 * 超长terms 查询
 * 突破1024限制
 */
public class MaxBatchQueryUtils {
    public static BoolQueryBuilder addTerms(List<String> ids, String key) {
        return addTerms(new HashSet<>(ids), key);
    }

    public static BoolQueryBuilder addTerms(Set<String> ids, String key) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
        List<List<String>> lists = fixedGrouping(new ArrayList<>(ids), 1024);
        for (List<String> list : lists) {
            boolQuery.should(QueryBuilders.termsQuery(key, list));
        }
        return boolQuery;
    }


    private static <T> List<List<T>> fixedGrouping(List<T> source, int n) {
        if (null == source || source.size() == 0 || n <= 0)
            return null;
        List<List<T>> result = new ArrayList<>();
        int remainder = source.size() % n;
        int size = (source.size() / n);
        for (int i = 0; i < size; i++) {
            List<T> subset = null;
            subset = source.subList(i * n, (i + 1) * n);
            result.add(subset);
        }
        if (remainder > 0) {
            List<T> subset = null;
            subset = source.subList(size * n, size * n + remainder);
            result.add(subset);
        }
        return result;
    }
}
